#!/usr/bin/python
# Filename: scisel.py

from numpy import *
from dtrange import *

# aim:
# get the indices of coords which occur in outcoords
# notes:
# - ordered data is needed for both coords and outcoords.
# - we expect that the data at the asked sample time (outcoords) exists in the original sample (coords). If not, a nan will be produced.

# crop = option whether the unavailable ones should be filled out with nan, or whether the unavailable ones
# should be ignored. In the latter, the output array will not have the same size as outcoords!


def cosel(coords = None, outcoords = None, crop=False):
    steps = len(outcoords)
    selout = repeat(None,steps)
    idx = 0
    outidx = 0
    for datetime in coords:
        if ((datetime >= outcoords[0]) & (datetime <= outcoords[steps - 1]) & \
            (outidx < steps)):
            if ((datetime >= outcoords[outidx])):
                while ((outidx < steps) & (outcoords[outidx] < datetime)):
                    outidx = outidx + 1
                if (datetime == outcoords[outidx]):
                    #outdata[outidx] = indata[idx]
                    selout[outidx] = idx
                outidx = outidx + 1
        idx = idx + 1
    if (crop == True):
        selout = selout[selout !=None]
    return selout

# someone may want to do this (if it's possible) instead of interpolation
def datacosel(indata, cocosel = None, coords = None, outcoords = None, \
              start= None,end= None,step = None,  \
              outstart = None, outend = None, outstep = None, \
              crop = False):
    ''' indata: input data
    '''
        
    if (cocosel == None):
        # if coordinates not given, make it from the start, end, step arguments
        if (coords == None):
            coords = dtrange(start, end, step)
        if (outcoords == None):
            if ((outstart != None) & (outend != None) ):
                if (outstep != None):
                    outcoords = dtrange(outstart, outend, outstep)
                else: 
                    outcoords = coords[(coords >= outstart) & (coords <=outend)]
            else:
                outcoords = coords
        cocosel = cosel(coords = coords, outcoords = outcoords)
    else:
        if (outcoords == None):
            outcoords = cocosel

    # remove NA (None) values if specified
    # would be more elegant if we could simply do cosel = cosel[cosel!=None], but None inside [] does strange things
    if (crop == True):
        tempcosel = []
        tempoutcoords = []
        for icosel,ecosel in enumerate(cosel):
            if (ecosel != None):
                tempcosel.append(ecosel)
                tempoutcoords.append(outcoords[icosel])
        cocosel = array(tempcosel)
        outcoords = array(tempoutcoords)

    dataout = tile(nan,len(cocosel))
    #dataout = indata[cosel[cosel!=None]]
    #dataout[cosel==None] = nan
    #dataout[cosel!=None] = indata[cosel[cosel!=None]]
    for iel, el in enumerate(cocosel):
        if (el != None):
            dataout[iel] = indata[el]
        else:
            dataout[iel] = None
    return(list([array(dataout), array(outcoords)]))


# # aim: build an array (boolean) selection from indices of any dimension
# # the coordinates from which elements of the multi-dim matrix are selected
# #      wrapper function for a more advanced array selection
# # SHAPE: shape of the multidim-matrix
# # SEL: list indices of each dimension of a matrix we want to selected. e.g. with SEL=[[0,1,2],[0],[0]],
# # we select a 3x1x1 matrix 
# # usage: datasel = data[arraysel([[0,1,2],[0,2,4],[22,23]],SHAPE=shape(data))
# def arraysel(SEL,SHAPE = None):
#     if (SHAPE == None):
#         SHAPE = []
#         for eSEL in SEL:
#             SHAPE.append(max(eSEL) + 1 )
#     lfirst = True
#     #ntruefalse: the matrix we want to construct, which can be used to select the data.
#     ntruefalse = True
#     for axidx,dimsize in reversed(list(enumerate(SHAPE))):
#         # build a boolean array for current dimension
#         curdimarray = (zeros(dimsize) == 1)
#         for element in SEL[axidx]: 
#             if (element != None): # workaround: data[None] selects all data
#                 curdimarray[element] = True 
#         otruefalse = ntruefalse
#         # for the first dimension, we initialize ntruefalse(shape)
#         if (lfirst):
#             ntruefalseshape = dimsize
#             lfirst = False
#         else:
#             ntruefalseshape = list(ntruefalse.shape)
#             ntruefalseshape.insert(0,dimsize)
#         ntruefalse = (zeros(ntruefalseshape) == 1)
#         for ielement,element in enumerate(curdimarray):
#             if (element == True):
#                 ntruefalse[ielement] = otruefalse
#             else:
#                 ntruefalse[ielement] = False
#     return ntruefalse
 



from numpy import *
import inspect
from operator import itemgetter

def procdf(function,allvars,procaxis,dimidx=0):
    iterate = False
    if (dimidx <= len(shape(allvars[0]))):
        if (procaxis[dimidx] == False):
            iterate = True

    if iterate:
        #procaxis[dimidx] == False  !!!
        #so  the first item shape of each variable allvars is allvars[0].shape[0] 
        dataout = []
        for axidx in range(allvars[0].shape[0]):
            allvarsin = []
            for evar in allvars:
                allvarsin.append(evar[axidx])
            dtotmp, outcoords = procdf(function,allvarsin,procaxis,dimidx=dimidx+1)
            dataout.append(dtotmp)
    else:
        allvarstemp = []
        for evar in allvars:
            # workaround for memory leak when just running  'dataouttemp = function(allvars)' in case of netcdf file references
            if type(evar) != ndarray:
                allvarstemp.append(evar)
            else:
                allvarstemp.append(array(evar))
            # end workaround

        # # if only one variable, then don't pass it as a list
        # if len(allvarstemp == 1):
        #     allvarstemp = allvarstemp[0]

        dataouttemp = function(allvarstemp)

        if (type(dataouttemp).__name__ == 'list'):
            dataout = dataouttemp[0]
            outcoords = dataouttemp[1]
            # vardimsout...
        else:
            dataout = dataouttemp
            shapeout = shape(dataout)
            # print 'shapeout', shapeout
            outcoords = []
            for eshape in shapeout:
                # <False> means that the dimension given by the output of the 
                # function is not specified. Afterwards, it should be checked 
                # whether this dimension is the same as the input dimension
                # If so, this should become None
                outcoords.append(False)
            # print 'outcoords', outcoords
    return dataout, outcoords

def multifunc(function,allvars,procaxis,vardims):
    """
    function: the function that has to be applied on the variable
    procaxis:
          Description: indicates for each variable (ranked from 0 to ... ) that 
                       have to be processed 
          Dimensions: the amount of variables occuring in any variable: 
          Example: (False, True, True, False, ...).
        
    vardims: 
          Description: list of 'pointers' to dimensions for each variable array 
          Dimensions: (amount of variables, dimensions of that variable) ,
                          so the second dimension is not fixed for each row
          Example: [[3,     1   , 2   , 4    , ...],
                    [2,     3   , 4   , 6    , ...],
                    ....                     
                   ]
    """
    # add dimensions to variables (i.e. expand array) 
    #in case they are missing (tbi: make this unnecessary)
    dimcount = len(procaxis) 
    for ivar,evar in enumerate(allvars):
        for idim in reversed(range(len(vardims[ivar]))): #reversed(range(dimcount)):
            if vardims[ivar][idim] == None: # vardims[ivar]:
                # if a function has to be applied on that dimension, 
                #only add an 'axis' for that dimension
                if procaxis[ivar] == True:
                    # print 1,ivar,  allvars[ivar].shape
                    allvars[ivar] = allvars[ivar][newaxis,:]
                    # print ivar,  allvars[ivar].shape
                # else: length of dimension is 
                # considered the same for all variables! 
                # so multiple copies are made from the dimension 
                # of an arbitrary other variable 
                # (we just take the first that has this variable)
                else:
                    ldone = False
                    for ivar2,evar2 in enumerate(allvars):
                        if idim in vardims[ivar2]:
                            if ldone == False:
                                # print 2,ivar, allvars[ivar].shape
                                # dimension length: evar2.shape[vardims[ivar2].index(idim) 
                                allvars[ivar] = repeat(allvars[ivar][newaxis,:],evar2.shape[vardims[ivar2].index(idim)],0)
                                # print ivar,  allvars[ivar].shape
                            ldone = True
                vardims[ivar][idim] = -1
                for idim2 in range(len(vardims[ivar])): #reversed(range(dimcount)):
                    if vardims[ivar][idim2] != None:
                        vardims[ivar][idim2] = vardims[ivar][idim2] + 1

                    
    # put dimensions of all variables arrays in the same order 
    # as the standard order (which is implicitely given by vardims)

    # for ivar,evar in enumerate(allvars):
    #     print evar.shape, vardims[ivar]
    for ivar,evar in enumerate(allvars):
        allvars[ivar] = transpose(evar,vardims[ivar])
    # for ivar,evar in enumerate(allvars):
    #     print 'shape before transposition:',evar.shape, vardims[ivar]
    #     print 'procaxis:', procaxis


    # if a certain dimension has
    for idim in range(len(procaxis)):
        # get maximum dimension
        maxdim = 0
        for ivar,evar in enumerate(allvars):
            maxdim = max(shape(evar)[idim],maxdim)
        for ivar,evar in enumerate(allvars):
            if ((shape(evar)[idim] == 1) & (maxdim > 1)):
                # print maxdim,idim
                allvars[ivar] = repeat(allvars[ivar],maxdim,idim)
                # tmpdim = range(len(procaxis))
                # tmpdim[idim] = 0
                # print 'tmpdim 1', tmpdim
                # for idim2 in range(idim):
                #     tmpdim[idim2] = tmpdim[idim2] + 1
                # print 'tmpdim', tmpdim
                # allvars[ivar] = transpose(allvars[ivar],tmpdim)
                # allvars[ivar] = repeat(allvars[ivar][:],maxdim,0)
                # inv = range(len(tmpdim))
                # for idim2, edim2 in enumerate(tmpdim):
                #     inv[edim2] = idim2
                # allvars[ivar] = transpose(allvars[ivar],inv)

    # for ivar,evar in enumerate(allvars):
    #     print 'shape after expansion:',evar.shape, vardims[ivar]
    #     print 'procaxis:', procaxis

    
    # now transpose all matrices in a similar way in such a way that the dimensions 
    # to be processed are at the end
    
    refsorted = sorted(zip(procaxis,range(dimcount)), key=itemgetter(0,1)) 
    trns = [] # the transpose indices
    trnsprocaxis = [] # procaxis after transpose
    #print('data processing started')
    for irefsorted,erefsorted in enumerate(refsorted):
        trns.append(erefsorted[1])
        trnsprocaxis.append(erefsorted[0])
    for ivar,evar in enumerate(allvars):
        allvars[ivar] = transpose(evar,trns)
    # ...

    # for ivar,evar in enumerate(allvars):
    #     print 'shape after transposition:',evar.shape

    dataouttrns,outcoordstrnstmp = procdf(function,allvars,trnsprocaxis)

    # print 'dataouttrns',shape(dataouttrns)


    #print('shapedataouttrns', shape(dataouttrns), outcoordstrnstmp)
    # 
    # # in case the function introduces extra dimensions: Add these extra dimensions in front of the
    # # processed dimensions (we accually put them at the end here but this doens't matter

    # amount of processed input dimensions
    procdim = len(where(array(trnsprocaxis) == True)[0])
    # amount of non-processed input dimensions
    nonprocdim = len(where(array(trnsprocaxis) == False)[0])
    # amount of extra dimensions introduced by the function
    extradims = len(shape(dataouttrns)) - len(trnsprocaxis)
    # print('extradims',extradims)
    trnsoutaxis = list(trnsprocaxis)


    # from procdf, the extra dimension is assumed in front of the processed input dimensions 
    # (and after the previous extra dimensions). It is put in front of all output dimensions

    # We put each extra dimension in front of all the others for the output
    # so we raise each reference by the number of extra dimensions
    for i in range(extradims):
        for itrns, etrns in enumerate(trns):
            trns[itrns] = trns[itrns]+1
    for i in range(extradims):
        trnsoutaxis.append(True)
        # from procdf, the extra dimension is assumed in front of the processed input dimensions 
        # (and after the previous extra dimensions). It is put in front of all output dimensions
        trns.insert(nonprocdim+i,i)

    
    # Reorganize the output coordinate list, so that they 
    # are aligned with the standard dimension numbering. So add 'nones' where needed
    outcoordstrns = []
    iocttmp = 0
    for i,etrnsoutaxis in enumerate(trnsoutaxis):
        if etrnsoutaxis: # if the function is applied along the current dimension
            outcoordstrns.append(outcoordstrnstmp[iocttmp])
            iocttmp = iocttmp + 1
        else:
            # <None> means that the current dimension corresponds to the input data 
            # dimensions (is trivially none when this dimension isn't processed)
            outcoordstrns.append(None)
    
    #print('data processing ended')
    
    # build the 'inverse permutation operator'
    inv = range(len(trns))
    for itrns, etrns in enumerate(trns):
        inv[etrns] = itrns
    
    # inverse permutation of the output data
    # print dataout.shape, inv
    dataout = transpose(dataouttrns,inv)
    
    outcoords = []
    # inverse permuation of the output coordinates
    for iinv,einv in enumerate(inv):
        outcoords.append(outcoordstrns[einv])
    
    return dataout,outcoords




# calculate mean
from numpy import *
from dtrange import *
from scisel import *

# 1. [2. 3. 4.] 
#average(


def avgcycle(data, timeco = None, timewarp = None, cclose=False,skipnan=False):
    """ calculate averaged cycle
input:
   data: input data
   timeco: data coordinates
   timewarp: length of the cycle
   skipnan: skip nan-values
option:
   cclose: add an additional point at the end identical the beginning to close the cycle
 todo: add an option to count the amount of samples for each member of the cycle -> then crop in cosel should
 be set to True so that we still take the mean even though there is no data available.
"""
    if (timeco == None):
        timeco = arange(len(data))
    if (timewarp == None):
        timewarp = timeco[1] - timeco[0]
    iscounted = tile(False,len(data))
    tempco = dtrange(timeco[0],timeco[len(timeco)-1],timewarp)
    dataout = []
    timecoout = []
    for idata in range(len(data)):
        if (iscounted[idata] == False):
            tempco2 = tempco + (timeco[idata] - timeco[0])
    
            #coordinates of data taken into account for this timestamp
            tempcosel = cosel(timeco,tempco2)
    
            # include some option to ignore the times on which there is no data
            # include some option to ignore the nan data
    
            # if some data could not be selected, a nan will be produced
            # in cosel, crop is left to False, because we want to discover whenever data is not available
            tdatacosel = datacosel(data,cocosel = tempcosel)[0]

            # we don't want to end up with an error if None is met: we just want a 'nan' from the mean.
            for edata in tdatacosel:
                if (edata == None):
                    edata = nan
            if skipnan:
                tdatacosel = array(tdatacosel)
                tdatacosel = tdatacosel[where(isnan(tdatacosel) == False)]
            dataout.append(mean(tdatacosel))
            timecoout.append(timeco[idata])
            #dataout.append(mean(scisel(data,timeco,tempco2) )
    
            #it would be much more elegant if we could simple do: iscounted[tempcosel] = True
            #but None inside [] does strange things,
            #          (tempcosel!=None : manually crop (same as if we did cosel(...,crop=True)))
            for etempcosel in tempcosel:
                if (etempcosel != None): # None inside [] does strange things
                    iscounted[etempcosel] = True
        # add an additional point to close the cycle
    if (cclose == True):
        timecoout.append(timeco[0]+ timewarp)
        dataout.append(dataout[0])
    return(list([array(dataout),array(timecoout)]))    
    
# calculate deviation from mean
def anomaly(x):
    y = zeros(len(x))
    meanx = mean(x)
    for ix,ex in enumerate(x):
        y[ix] = ex -meanx
    return (y)



def avgglide(x,avlen):
    xout = np.zeros_like(x)
    xwork = np.zeros_like(x)
    if np.mod(avlen,2) == 0:
        xwork[-1] = np.nan
        xwork[:-1] = (x[1:] + x[:-1])/2.
    else:
        xwork = x
    lenxout = xout.shape[0]
    avlen2 = int(avlen/2.)
    xout[:avlen2] = np.nan
    xout[-avlen2:] = np.nan
    for i in range(0,avlen,1):
        xout[avlen2:(-avlen+avlen2)] = xout[avlen2:(-avlen+avlen2)] + xwork[i:(-avlen+i)]
    return xout/avlen

    # # the slow way
    # for j in range(len(x)):
    #     xout[j] = np.mean(x[max(0,(j-int(avlen/2.))):min((j+int(avlen/2.)),lenxout)],axis=0)
    # return (xout,)




#!/usr/bin/python
# Filename: __init__.py

from dtrange import *

# selection procedures
from scisel import *

# apply a function repeatedly on a multidimensional matrix
from scimulti import *

# statistics calculation
from scistat import *

# various
from sciop import *



from numpy import zeros,nan
import numpy as np
import math as mt

# def getidx(idxs,numfac):

def steinalp(f,numfac):
    '''  purpose: factor seperation procedure following Stein and Alpert (1993), Factor
    Seperation in Numerical Simulations (JAS). Example will be added soon (2013-11-25).

    f: data set with factors
    numfac: amount of factors investigated -> The function f needs to have 2**numfac elements.
    '''

    #a few preparations
    frav = np.array(f)
    fshp = frav.shape #np.shape(frav)[:numfac]
    nmshape = 2**numfac
    lshf = numfac
    
    ishape = 0
    while nmshape > 1:
        nmshape = nmshape / frav.shape[ishape]
        ishape = ishape + 1
    newshape = [2**numfac] 
    if len(frav.shape) > ishape:
        newshape = newshape +list(frav.shape[ishape:])

    frav.shape = newshape
    fsep = np.zeros(newshape)
    
    # here we go...
    fsep[0] = np.array(frav[0])
    for posfsep in range(1,len(fsep)):
    
        fsep[posfsep] = np.array(frav[posfsep])
        idxs = []
        posfseptmp = int(posfsep)
    
        # we discover the indices from which we take combinations
        for ishp in reversed(range(lshf)):
            if np.mod(posfseptmp,2) == 1:
                idxs.append(ishp)
                posfseptmp = (posfseptmp - 1)
            posfseptmp = posfseptmp/2
    
        idxs.sort()
    
        l = len(idxs)
    
        # a loop over the different kind of combinations :
        # 0 1 2 3...m -- ... l
        # l l l l   ...      l
        for m in range(1,l):
            # determine the number of possible combinations of the current kind
            numcomb = mt.factorial(l)/mt.factorial(m)/mt.factorial(l-m)
    
            # now we make the combinations of the indices
            pos = range(m) # start position (e.g. (1,2,3,4)
            if pos != []:
                for inumcomb in range(numcomb):
                
                    curidxs = []
                    for ipos,epos in enumerate(pos):
                        # the current matrix index
                        curidxs.append(idxs[epos])
    
                    posf = 0
                    for i in range(lshf):
                        if i in curidxs:
                            posf = posf + 2**(lshf-i-1)
                    
                    fsep[posfsep] = fsep[posfsep] +  (-1)**(l-m) * frav[posf]
                    pos[-1] = pos[-1] +1 
                    for ipos,epos in reversed(list(enumerate((pos)))):
                        if (ipos >= 1) and (epos == l):
                            pos[ipos - 1] = pos[ipos -1] + 1
                            pos[ipos] = 0 #pos[ipos] + 1
        fsep[posfsep] = fsep[posfsep] + (-1)**l * frav[0]
        
    fsep.shape = fshp
    if type(f).__name__ == 'list':
        return list(fsep)
    elif type(f).__name__ == 'tuple':
        return tuple(fsep)
    else:
        # just array?
        return fsep

def tsfill(valin,dtin,dtout):
    ''' purpose: regrid timeseries... fill missing timesteps in a timeseries with nan values, or leave out unwanted timesteps
    valin: time series values
    dtin: time steps of the timeseries
    dtout: desired output timesteps
    '''

    valout = zeros(len(dtout))*nan
    lendtin = len(dtin)
    idtin = 0
    for idt,edt in enumerate(dtout):
        while((dtin[idtin] < edt) & (idtin < (len(dtin)-1))):
            idtin = idtin + 1
    
        if edt == dtin[idtin]:
            valout[idt] = valin[idtin]
    return valout

def mergedata(x):
    x1 = x[0]
    x2 = x[1]
    xout = zeros((len(x1)+len(x2))) 
    for ix1,ex1 in enumerate(x1):
        xout[ix1] = ex1
    for ix2,ex2 in enumerate(x2):
        xout[ix2+len(x1)] = ex2

    return xout





# aim: 
# scifunc: apply a certain function serveral times on a multidimensional matrix; calculate a certain
# function along a specified axis
# reddim: reduce the dimension of a multidimensional matrix


# todo for scifunc: 
# pass multiple variables for one function
# make an array attribute as a substitute for arraysel
# currently, it is expected that the function conserves the # of dimensions. If not, there will be problems.
#     so possibility to decrease dimension of matrix is still needed in which we may want to throw away a dimension automatically (like
#     for making averages by summing over an index for a certain dimension) 
#     we should detect automatically when it has to be removed, e.g. when the function reduces the # of dimensions of a matrix
#     
# coordinate build in as a array attributeis: 
#     array.arraysel(SEL) 
#     arraycrop (of zoiets)
# 

from numpy import *
from sciproc import arraysel

# apply a function for each element along various axis
# data: matrix to be processed
# axisref: a list of boolean which indicates along which axis the function has to be applied
#       if false: the function is applied seperately for each element along the current axis
#       if true: all elements of the current axis are used as input for the function
# example, if data is a 4x3x2 matrix and SEL=[False,True,False], the function deffunc will be applied
# for each of the 4 elements along the first axis and for each of he 2 elements along the third axis.
# So the function will be applied 4x2 times. Each time, the 3 elements along second axis are
# all passed to the function. The size of the output matrix depends on the function.
def multifunc(data,axisref,deffunc):
    #enige wat we eigenlijk nodig hebben voor scandata is shape(data) en shape(dataout) en dataout
    dataout = zeros(sizeout(data,axisref,deffunc)[0])
    SEL = list(repeat(None,len(shape(data))))
    SELout = list(repeat(None,len(shape(dataout))))

    # todo: dataout should be able to be of arbitrary dimension and not necessarily the size of datain!
    scandata(0,axisref,SEL,data,deffunc,SELout,dataout,array(True,dtype='bool'),array(True,dtype='bool'))
    return dataout

# this function is called by multifunc. It iterates through the necessary dimensions to apply a function
def scandata(dimidx,axisref,SEL,data,deffunc,SELout,dataout,arraySEL,arraySELout):
    # we lopen elk element van een as af
    axidx = 0
    while (((axisref[dimidx] == True) & (axidx == 0)) | \
            ((axisref[dimidx] == False) & (axidx < size(data,axis=dimidx)) )):
        if (axisref[dimidx] == False):
            SELout[dimidx] = [axidx]
            SEL[dimidx] = [axidx]
            #this should be simplified further!
        else:
            SELout[dimidx] = range(size(dataout,axis=dimidx))
            SEL[dimidx] = range(size(data,axis=dimidx))
        
        larraySELout = repeat(False,size(data,axis=dimidx))
        larraySELout[SELout[dimidx]] = True

        larraySEL = repeat(False,size(data,axis=dimidx))
        larraySEL[SEL[dimidx]] = True
        if dimidx > 0:
            larraySELout = transpose(transpose(arraySELout)[newaxis])*larraySELout[newaxis]
            larraySEL = transpose(transpose(arraySEL)[newaxis])*larraySEL[newaxis]

        # we gaan naar een hogere dimensie
        if (dimidx < (len(shape(data)) - 1)):
            scandata(dimidx + 1,axisref,SEL,data,deffunc,SELout,dataout,larraySEL,larraySELout)
        else:
            dtemp = deffunc(array([20.0,10.0]))
            # dtempin = data[larraySEL]
            # SHAPE = []
            # for iSHAPE in range(len(SEL)):
            #     if (axisref[iSHAPE] == True):  
            #         SHAPE.append(len(SEL[iSHAPE]))
            # dtempin.shape = SHAPE

            # dtemp= deffunc(dtempin)

            # if ((type(dtemp).__name__ == 'list') or (type(dtemp).__name__ == 'tuple')):
            #     dtemp = dtemp[0]

            print('SEL:',SEL)
            print('SELout:',SELout)
            # tricky!
            #dataout[larraySELout] = dtemp.ravel()

        axidx = axidx + 1

# reduce dimension of a multidimensional matrix
# input: 
#   data: matrix
#   indices: th dimension that has to be left away
#   el: which element has to be taken for the to be removed dimension (default: first element)
def reddim(data,indices,el = 0):
    #el = repeat(0,len(indices))
    for dim in range(len(shape(data))):
        if (dim in indices):
            SEL[dim] = el
        else:
            SEL[dim] = range(size(data,axis=dim))
    return data[arraysel(SEL,shape(data))]

# also called by multifunc, but may be used for other things also. 
# ----Isn't this just doing what's done at the inner scandata loop? maybe it has to be merged with that to have one
# single logic...---
def sizeout(data,axisref,deffunc):
    SEL = list(repeat(None,len(shape(data))))
    sizeout = list(repeat(None,len(shape(data))))
    #before calculating the dataout matrix, its dimension has to be acquired
    for dim in range(len(shape(data))):
        if (axisref[dim] == True):
            SEL[dim] = range(size(data,axis=dim))
        else:
            SEL[dim] = [0] 
            sizeout[dim] = size(data,axis=dim)

    # the first item in this list contains dummy data which will not be used; 
    # the other items potentially contain the output coordinates if available

    # print(shape(data[arraysel(SEL,data.shape)]))

    dtempin = data[arraysel(SEL,data.shape)]
    SHAPE = []
    for iSHAPE in range(len(SEL)):
        if (axisref[iSHAPE] == True):  
            SHAPE.append(len(SEL[iSHAPE]))
    dtempin.shape = SHAPE

    dummydataout = deffunc(dtempin)

    dimout = 0
    for dim in range(len(shape(data))):
        if (axisref[dim] == True):
            # if list, get the output data size which is given by the first item in the list
            # if not list (no output coordinates given), just get the size of the data
            if ((type(dummydataout).__name__ == 'list') or (type(dummydataout).__name__ == 'tuple')):
                sizeout[dim] = size(dummydataout[0],axis=dimout)
            else:
                sizeout[dim] = size(dummydataout,axis=dimout)
            dimout = dimout + 1

    # shapedataout = sizeout(ncin.variables[evariable][:],\
    #                        axisref, \
    #                        lambda data: deffunc(data) )
    # get the output coordinates of the processed dimensions and save it as a seperate list
    if ((type(dummydataout).__name__ == 'list') or (type(dummydataout).__name__ == 'tuple')):
        if (type(dummydataout[1]).__name__ == 'list' ): # multiple coordinate set stored as a list
            outcoords = dummydataout[1] 
        else:
            if (dummydataout[1] != None):
                outcoords = list(dummydataout[1])
            else:
                outcoords = None
    else:
        outcoords = None # function doesn't have output coordinates available

    # sizeout: size of the output data if one would apply multifunc
    # outcoords: list of coordinate sets of each dimension where axisref == True
    return sizeout, outcoords



# # reduce multiple dimensions at ones
# # indices: list of indices
# # data: the multi-dim matrix we want to reduce
# # todo: rewrite it to make it more efficient
# def reddimmulti(data,indices):
#     dataout = data
#     for index in range(len(indices)):
#         dataout = reddim(dataout,indices[index])
#         #we have to correct the indices as dimension of the output matrix is reduced already
#         for index2 in range(index,len(indices)):
#             indices[index2] = indices[index2] - 1
#     return dataout




from datetime import datetime, timedelta

def dtrange(*args):
    # purpose: alternative for range() that works with datetimes
    if len(args) != 3:
        return range(*args)
    start, stop, step = args
    if start < stop:
        cmp = lambda a, b: a < b
        inc = lambda a: a + step
    else:
        cmp = lambda a, b: a > b
        inc = lambda a: a - step
    #output = [start]
    output = []
    while cmp(start, stop):
        output.append(start)
        start = inc(start)

    return output
